define('smart-attachments/model/AttachmentItem',
    [
        'jquery',
        'underscore',
        'jira/lib/class',
        'smart-attachments/base/DNDHelper',
        'smart-attachments/base/Functions',
        'smart-attachments/base/Operations',
        'smart-attachments/AsyncModule',
        'smart-attachments/toolbox',
        'smart-attachments/dialog/RenameDialog',
        'smart-attachments/dialog/RemoveDialog',
        'smart-attachments/dialog/SendToEmail'
    ], function ($, _, Class, DNDHelper, Functions, Operations, AsyncModule, Toolbox, RenameDialog, RemoveDialog, SendToEmailDialog) {

        var fileViewSelectors = '[file-preview-type=image], [file-preview-type=document]';

        function addCommentIcon(attachment) {
            if (attachment.find('dd.attach-cat-comment').length === 0) {
                if (attachment.parent().hasClass('attach-images')) {
                    attachment.find('.attachment-size').before(AttachCategory.comment());
                } else {
                    attachment.find('.attachment-size').after(AttachCategory.comment());
                }
            }
        }

        function menuClickWrapper(originalFunction) {
            var args = Array.prototype.slice.call(arguments, 1);
            var event = args[0];
            event.stopPropagation();
            event.preventDefault();
            var result = originalFunction.apply(this, args);
            $(document).click();
            return result;
        }

        function isCommented(element) {
            var value = element.dataset['hasComments'];
            return (value && value.toLowerCase() === 'true');
        }

        var OpenDocumentsStorage = function (issueKey, documentId) {
            var storage = window.localStorage;
            var key = issueKey + ":" + documentId;

            function open() {
                storage.setItem(key, "1");
            }

            function close() {
                storage.removeItem(key);
            }

            function isOpen() {
                return storage.getItem(key) === "1";
            }

            return {
                open: open,
                close: close,
                isOpen: isOpen
            }
        };

        var AttachmentItem = Class.extend({
            templateMenu: AttachCategory.attachmentOptionsMenu,
            templateVersionExpander: AttachCategory.versionExpander,

            init: function (attr) {
                _.extend(this, attr);
                this.isDocument = Functions.isDocument(this.element);
                if (this.isDocument) {
                    this.hasComments = _.some(this.revisions, isCommented);
                } else {
                    this.hasComments = isCommented(this.element);
                }
                this.openDocumentStorage = new OpenDocumentsStorage(this.issue.getIssueKey(), Functions.getDocumentId(this.element));
                var nameElement = this.element.querySelector('dl > dt > a');
                this.href = nameElement.getAttribute('href');
                this.attachmentName = this.isDocument ? Functions.getDocumentName(this.element) : nameElement.text;
                this.attachmentId = Functions.getAttachmentId(this.element);
                this.tempOfficeName = this.getAttachmentNameForEditInOffice(this.attachmentName);
                //декорируем копию.
                this.$node = $(this.element).clone();
                this.$node.find('a').attr('draggable', false);
                this.revisionViews = this.createRevisionViews(this.revisions);
                this.decorate();
            },

            createRevisionViews: function (revisions) {
                return revisions.map(function (revisionElement) {
                    var revisionView = new AttachmentItem({
                        owner: this,
                        containerId: this.containerId,
                        element: revisionElement,
                        hasCreatePermission: this.hasCreatePermission,
                        canAddComment: this.canAddComment,
                        hasComments: isCommented(revisionElement),
                        category: this.category,
                        revisions: [],
                        isRevision: true,
                        issue: this.issue
                    });
                    revisionView.bind('attachment:remove', function (event, item) {
                        this.removeRevision(item);
                        this.trigger('attachment:remove', item);
                    }.bind(this));
                    return revisionView;
                }, this);
            },

            getItemId: function () {
                return this.attachmentId;
            },

            getDocumentId: function () {
                if (this.isDocument) {
                    return Functions.getDocumentId(this.element);
                }
            },

            getMoveInfo: function () {
                var revisionElements = [];
                var attachmentIds = [];
                var isDocument = this.isDocument;
                if (isDocument) {
                    revisionElements = this.revisionViews.map(function (view) {
                        attachmentIds.push(view.getItemId());
                        return view.element;
                    });
                } else {
                    attachmentIds.push(this.getItemId());
                }
                return {
                    attachmentItem: this,
                    isDocument: isDocument,
                    documentId: this.getDocumentId(),
                    documentName: Functions.getDocumentName(this.element),
                    attachmentIds: attachmentIds.reverse(),
                    revisionElements: revisionElements
                };
            },

            removeRevision: function (revision) {
                this.revisions = _.filter(this.revisions, function (item) {
                    return Functions.getAttachmentId(item) !== revision.getItemId();
                });
            },

            isSelected: function () {
                return this.$node.hasClass('selected-attach');
            },

            showDeleteDialog: function (event) {
                event.stopPropagation();
                event.preventDefault();
                var dialog = new RemoveDialog({
                    attachmentId: this.attachmentId,
                    issue: this.issue,
                    isDocument: this.isDocument,
                    isRevision: this.isRevision,
                    attachmentName: this.attachmentName
                });
                dialog.bind('remove:complete', function () {
                    this.trigger('attachment:remove', this);
                }.bind(this));
                dialog.show();
                return true;
            },

            showRenameDialog: function () {
                var renameDialog = new RenameDialog({
                    attachmentId: this.attachmentId,
                    value: this.attachmentName,
                    issue: this.issue,
                    categoryId: this.category.id,
                    isDocument: this.isDocument,
                    documentId: this.getDocumentId()
                });

                renameDialog.bind('rename:complete', function (event, value) {
                    this.changeAttachmentTitle(value);
                    this.attachmentName = value;
                }.bind(this));

                renameDialog.show();
                return false;
            },

            showSendToEmailDialog: function () {
                var sendToEmailDialog = new SendToEmailDialog({
                    issue: this.issue,
                    items: [this]
                });

                sendToEmailDialog.show();
            },

            toggle: function () {
                if (this.$node.hasClass('hidden')) {
                    this.$node.removeClass('hidden');
                } else {
                    this.$node.addClass('hidden');
                }
            },

            toggleRevisions: function (event) {
                event.preventDefault();
                event.stopPropagation();
                if (this.revisionViews.length) {
                    var $item = $(event.delegateTarget);
                    if ($item.hasClass('collapse')) {
                        this.openDocumentStorage.open();
                        $item.removeClass('collapse').addClass('expand');
                    } else {
                        this.openDocumentStorage.close();
                        $item.removeClass('expand').addClass('collapse');
                    }
                    this.revisionViews.forEach(function (item) {
                        item.toggle();
                    })
                }
            },

            downloadDocument: function () {
                Operations.downloadFileFromPath(AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/download-doc?' +
                    'issueKey=' + this.issue.getIssueKey() +
                    "&catId=" + this.category.id +
                    "&docId=" + this.getDocumentId()
                );
                return false;
            },

            onMoveToCategory: function (event) {
                event.preventDefault();
                event.stopPropagation();
                var $item = $(event.delegateTarget);
                this.trigger('attachment:move', [this.getItemId(), this.category.id, $item.data('categoryId')]);
                $(document).click();
            },

            attachHandlers: function () {
                var ctx = this;
                this.$node.find('a.icon-delete, .aui-iconfont-delete').click(this.showDeleteDialog.bind(this));
                this.$node.find('a.add-attach-comment').click(this.startCommentThread.bind(this));
                this.$node.find('a.rename-attach').on('click', _.wrap(this.showRenameDialog, menuClickWrapper).bind(this));
                this.$node.find('a.sa-mail-attach').on('click', _.wrap(this.showSendToEmailDialog, menuClickWrapper).bind(this));
                if (this.isDocument) {
                    var expander = this.$node.find('.attach-cat-version-expander-container');
                    expander.click(this.toggleRevisions.bind(this));
                    if (this.openDocumentStorage.isOpen()) {
                        expander.trigger('click');
                    }
                    this.$node.find('a.download-doc').click(_.wrap(this.downloadDocument, menuClickWrapper).bind(this));
                }
                if (!this.isRevision) {
                    this.$node.find('a.load-version').click(this.loadVersion.bind(this));
                    this.$node.find('a.move-to-cat').click(this.onMoveToCategory.bind(this));
                    this.$node.find('div.sa-invisible-panel-wrap').on('click', function (event) {
                        var $target = $(event.target);
                        $target.closest('li').toggleClass('selected-attach');
                    });
                    this.attachGoEditHandler();
                } else {
                    this.$node.attr('draggable', false);
                }

                if (Toolbox.isFVIModuleEnabled()) {
                    var fviItems = this.$node.find(fileViewSelectors);
                    fviItems.one('mousedown.sa-comm-fv', function (event) {
                        var events = $(event.delegateTarget).data("events").simpleClick;
                        if (events !== undefined && events.length < 2) {
                            fviItems.on("simpleClick.fvOpen", function () {
                                FVI.initButtons(ctx.getItemId());
                            });
                        }
                    })
                }
            },

            loadVersion: function () {
                var attachmentId = this.getItemId();
                var versionInfo = {
                    parent: attachmentId,
                    name: this.attachmentName,
                    catId: this.category.id,
                    isDoc: !!this.isDocument,
                    callerAttachId: attachmentId
                };
                if (this.isDocument) {
                    versionInfo.documentId = this.getDocumentId();
                }
                this.trigger('attachment:prepareForRevision', [versionInfo]);
            },

            attachGoEditHandler: function () {
                AsyncModule.onReady('goedit', this.enableGoEditMenu.bind(this));
                this.$node.find('.sa-goedit-integration a').on('click', function (event) {
                    event.preventDefault();
                    var id = this.getItemId();
                    var attachName = this.attachmentName;
                    SmartAttachAnalytics.eventEditWithGoEdit(this.href);
                    $(document).trigger('goedit:open-file', [{"fileID": id, "fileName": attachName}]);
                }.bind(this));
            },

            enableGoEditMenu: function () {
                console.log("Enable Goedit for attachmentId: " + this.getItemId());
                $(document).trigger('goedit:open-popover', {
                    element: this.$node.find('.attachment-title a, a.attachment-title').get(0),
                    fileID: this.getItemId(),
                    fileName: this.attachmentName
                });

                this.$node.find('.sa-goedit-integration').show();
            },

            getAttachmentNameForEditInOffice: function (attachmentName) {
                var value = attachmentName.trim().replace('#', '');
                return encodeURIComponent(EditInOffice.makeTempAttachmentNameForURL(value));
            },

            makeAttachmentOptionsMenu: function () {
                var categoryId = this.category.id;
                var filteredCategories = _.filter(this.categories, function (category) {
                    return category.id !== categoryId;
                });
                return this.hasCreatePermission ?
                    this.templateMenu({
                        id: this.$node.data('attachmentId'),
                        isDocument: this.isDocument,
                        isRevision: this.isRevision,
                        categories: filteredCategories,
                        issueId: this.issue.getIssueId(),
                        canAddComment: this.canAddComment,
                        officeType: EditInOffice.getType(this.href),
                        name: this.tempOfficeName,
                        cat: this.category.id,
                        issueKey: this.issue.getIssueKey(),
                        isSupported: EditInOffice.isSupportedBrowser(),
                        user: AJS.params.loggedInUser
                    }) : null;
            },

            decorateMenu: function () {
                var menu = this.makeAttachmentOptionsMenu();

                var deleteActions = this.$node.find('.attachment-delete');
                if (!deleteActions.length) {
                    deleteActions = $('<dd class="attachment-delete"/>');
                    this.$node.find('.attachment-title').before(deleteActions);
                }

                deleteActions = this.$node.find('.attachment-delete').clone();

                var attachHeaderLine = $('<div class="sa-attach-header-line"></div>');
                this.$node.find('dl').append(attachHeaderLine);
                attachHeaderLine.append(deleteActions);
                if (menu) {
                    var $menu = $(menu);
                    this.$node.find('.attachment-delete').prepend($menu);
                    attachHeaderLine.append($menu);
                }
            },

            changeAttachmentTitle: function (attachmentName) {
                var titleLink = this.$node.find('dt a.attachment-title');
                if (titleLink.length === 0) {
                    titleLink = this.$node.find('dt.attachment-title a');
                }
                if (titleLink.length) {
                    var currentTitle = titleLink.attr('title');
                    var parts = currentTitle.split('-');
                    parts[0] = attachmentName;
                    titleLink.html(attachmentName);
                    titleLink.attr('title', parts.join('-'));
                    titleLink.attr('file-preview-title', attachmentName);
                }
            },

            startCommentThread: function (event) {
                event.preventDefault();
                FVI.showFVAndCreateThread(this.getItemId(), this.$node);
            },

            viewCommentThread: function () {
                var attachmentItem = this;
                if (this.isDocument) {
                    var revisionView = _.find(this.revisionViews, function (revisionView) {
                        return revisionView.hasComments;
                    });
                    if (revisionView) {
                        attachmentItem = revisionView;
                    }
                }
                FVI.showFVAndSidebar(attachmentItem.getItemId(), attachmentItem.$node);
            },

            attachComments: function () {
                if (this.canAddComment) {
                    addCommentIcon(this.$node);
                    this.$node
                        .find('.attach-cat-comment a.aui-iconfont-add-comment')
                        .click(this.startCommentThread.bind(this));
                }

                this.$node
                    .find('.attach-cat-comment a.aui-iconfont-comment')
                    .click(this.viewCommentThread.bind(this));

                if (this.hasComments) {
                    this.$node
                        .find('dd.attach-cat-comment')
                        .addClass('hasComment');
                }
            },

            convertImageToFile: function () {
                var dt = this.$node.find('dt');
                dt.find('.blender').remove();
                dt.after(this.$node.find('dd.attachment-date'));

                if (this.$node.find('.icon-delete, .aui-iconfont-delete').length) {
                    dt.after("<dd class='attachment-delete'></dd>");
                    this.$node.find('dd.attachment-delete').append(dt.find('.attachment-delete').removeClass('attachment-delete'));
                } else {
                    var deleteItem = dt.find('dd.attachment-delete');
                    deleteItem.append('<span class="icon"></span>');
                    dt.after(deleteItem);
                }

                this.$node.find('dd.attachment-size').after(this.$node.find('dd.attach-cat-comment'));
            },

            setDataTransfer: function (dataTransfer) {
                if (dataTransfer) {
                    dataTransfer.effectAllowed = 'move';
                    var data = {
                        attachmentId: this.getItemId(),
                        categoryId: this.category.id
                    };
                    dataTransfer.setData('text', JSON.stringify(data));
                    localStorage.setItem("dndItem", "attachment");
                }
            },

            setDataTransferAttach: function (dataTransfer, dragPosition) {
                this.setDataTransfer(dataTransfer);
                if (dataTransfer && 'function' === typeof DataTransfer.prototype.setDragImage) {
                    dataTransfer.setDragImage(DNDHelper.createDragImage(this.$node), dragPosition.x, dragPosition.y);
                }
            },

            decorateDocument: function () {
                this.$node.prepend(this.templateVersionExpander({id: this.getDocumentId()}));
            },

            decorateRevision: function () {
                this.$node.addClass('revision hidden');
            },

            decorateBulk: function () {
                this.$node.prepend(AttachCategory.selectionArea());
            },

            decorateDnD: function () {
                if (!this.isRevision) {
                    this.$node.attr('draggable', true);
                    this.$node.on('dragstart', this.onDragStart.bind(this));
                    this.$node.on('dragend', function () {
                        localStorage.removeItem("dndItem");
                    });
                }
            },

            hideGoeditPopup: function () {
                var wrapper = document.querySelector('#goedit-popover-wrapper');
                if (wrapper && wrapper.hasChildNodes()) {
                    _.each(wrapper.childNodes, function (node) {
                        wrapper.removeChild(node);
                    });
                }
            },

            onDragStart: function (event) {
                this.hideGoeditPopup();
                event.stopPropagation();
                if (this.$node.attr('draggable') !== 'true') {
                    event.preventDefault();
                    return false;
                }
                var $attachment = this.$node;
                DNDHelper.setDataTransferAttachmentDownloadURL(event, $attachment);
                this.setDataTransferAttach(event.originalEvent.dataTransfer, DNDHelper.getDragPosition(this.$node, event));
            },

            decorate: function () {
                if (this.isDocument) {
                    this.decorateDocument();
                }
                if (this.isDocument || this.isRevision) {
                    if (this.containerId === 'attachment_thumbnails') {
                        this.convertImageToFile();
                    }
                    if (this.isRevision) {
                        this.decorateRevision();
                    }
                }
                this.decorateBulk();
                this.decorateDnD();
                this.decorateMenu();
                this.changeAttachmentTitle(this.attachmentName);
                this.attachComments();
                this.attachHandlers();
            },

            isHaveRevisions: function () {
                return this.revisions.length > 0;
            },

            remove: function (removeSourceElement) {
                this.$node.remove();
                if (removeSourceElement) {
                    $(this.element).remove();
                }
                if (this.isDocument) {
                    this.revisionViews.forEach(function (view) {
                        view.remove(removeSourceElement);
                    });
                }
            },

            render: function () {
                return this.$node;
            }
        });

        return AttachmentItem;
    });
