define('smart-attachments/dialog/SendToEmail',
    [
        'jquery',
        'underscore',
        'jira/lib/class',
        'smart-attachments/toolbox'
    ],
    function ($, _, Class, Toolbox) {
        var REG_EXP = /^([a-zA-Z0-9_\.\-])+@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/;
        var EmailChecker = {
            valid: function (email) {
                return REG_EXP.test(email);
            },

            testEmails: function (string) {
                var result = {
                    success: true,
                    emailsCount: 0
                };
                $.each($.trim(string).split(/\s*,\s*/), function (i, v) {
                    if ($.trim(v).length > 0) {
                        result.emailsCount++;
                        if (!EmailChecker.valid(v)) {
                            result.success = false;
                        }
                    }
                });
                return result;
            }
        };

        var DIALOG_CONFIG = {
            width: 650,
            height: 550,
            id: "mail-attach-dialog",
            closeOnOutsideClick: false
        };

        var SADialogEmail = Class.extend({
            init: function (opts) {
                _.extend(this, opts);
                this.projectKey = Toolbox.getProjectKey(this.issue.getIssueKey());
                this.createDialog();
            },

            createDialog: function () {
                var attachments = this.items.map(function (attachmentItem) {
                    return {
                        id: attachmentItem.getItemId(),
                        name: attachmentItem.attachmentName
                    }
                });

                var body = $(AttachCategory.mailDialog({
                    attachments: attachments,
                    issueKey: this.issue.getIssueKey(),
                    issueSummary: $('#summary-val').text()
                }));

                var emailDialog = new AJS.Dialog(DIALOG_CONFIG);

                emailDialog.addHeader(AJS.I18n.getText("attach.cat.mail.dialog.title"));
                emailDialog.addPanel('panel', body, 'mail-attach-dialog-panel');
                emailDialog.addButton(AJS.I18n.getText("attach.cat.mail.dialog.send"), this.onSend.bind(this), 'send-attach-email');
                emailDialog.addLink(AJS.I18n.getText("attach.cat.rename.dialog.cancel"), function (dialog) {
                    dialog.remove();
                }, '#');

                this.$subjectInput = body.find('#subject');
                this.$messageInput = body.find('#message');
                this.$putToComments = body.find('#putEmailToComments');
                this.$putToComments.on('change', this.storeCheckboxStateOfCommenting.bind(this));
                this.$toInput = body.find('#to');
                this.$ccInput = body.find('#cc');
                this.$bccInput = body.find('#bcc');
                _.each([this.$toInput, this.$ccInput, this.$bccInput], SmartAttachAutocomplete.init.bind(SmartAttachAutocomplete));
                this.$body = body;
                this.dialog = emailDialog;
            },

            onSend: function () {
                if (!this.checkEnteredMails(this.$body)) {
                    return false;
                }

                var attachmentIdsForSend = this.getFormCheckedAttachments(this.$body);

                JIRA.SmartAjax.makeRequest({
                    type: 'POST',
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/send/mail',
                    data: {
                        ids: attachmentIdsForSend,
                        issueKey: this.issue.getIssueKey(),
                        subject: this.$subjectInput.val(),
                        message: this.$messageInput.val(),
                        to: this.$toInput.val().trim(),
                        cc: this.$ccInput.val().trim(),
                        bcc: this.$bccInput.val().trim(),
                        sendWithoutAttachments: false,
                        copyOfEmailToComments: this.$putToComments.is(':checked')
                    },
                    success: function () {
                        this.gaAnalyticsAttachmentsSent(attachmentIdsForSend);
                        this.dialog.remove();
                        this.showSuccessMessageFlag();
                        if (this.$putToComments.is(':checked')) {
                            JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()])
                        }
                    }.bind(this),
                    error: this.showErrorMessageFlag
                });
            },

            show: function () {
                this.synchronizeCheckboxAndStoredStateOfCommenting(this.$putToComments).then(
                    function () {
                        this.dialog.show();
                    }.bind(this)
                );
            },

            getFormCheckedAttachments: function (formBody) {
                var markedAttachments = [];
                formBody.find('.attach:checked').each(function () {
                    markedAttachments.push($(this).attr('id'));
                });
                return markedAttachments;
            },

            checkEnteredMails: function (form) {
                $('.description', form).hide();

                var resultTo = EmailChecker.testEmails(this.$toInput.val());
                var resultCc = EmailChecker.testEmails(this.$ccInput.val());
                var resultBcc = EmailChecker.testEmails(this.$bccInput.val());

                if (resultTo.emailsCount === 0) {
                    $('#empty-to', form).show();
                } else if (!resultTo.success) {
                    $('#error-to', form).show();
                }

                if (!resultCc.success) {
                    $('#error-cc', form).show();
                }

                if (!resultBcc.success) {
                    $('#error-bcc', form).show();
                }

                var resultMessage = $.trim(this.$messageInput.val());
                if (!resultMessage) {
                    $('#empty-message', form).show();
                }

                var resultSubject = $.trim(this.$subjectInput.val());
                if (!resultSubject) {
                    $('#empty-subject', form).show();
                }

                return resultTo.success &&
                    resultTo.emailsCount > 0 &&
                    resultCc.success &&
                    resultBcc.success &&
                    resultMessage &&
                    resultSubject;
            },

            showSuccessMessageFlag: function () {
                JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.mail.dialog.success"), {
                    type: JIRA.Messages.Types.SUCCESS,
                    closeable: false,
                    timeout: 3
                });
            },

            showErrorMessageFlag: function () {
                JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.mail.dialog.error"), {
                    type: JIRA.Messages.Types.ERROR,
                    closeable: false,
                    timeout: 3
                });
            },

            gaAnalyticsAttachmentsSent: function (attachmentIds) {
                if (attachmentIds.length > 1) {
                    SmartAttachAnalytics.eventBulkOpSendEmail(attachmentIds.length);
                } else {
                    SmartAttachAnalytics.eventAttachmentSendToEmail();
                }
            },

            storeCheckboxStateOfCommenting: function () {
                JIRA.SmartAjax.makeRequest({
                    type: 'POST',
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/settings/email-to-comment',
                    data: {
                        projectKey: this.getProjectKey,
                        state: this.$putToComments.is(':checked')
                    }
                });
            },

            synchronizeCheckboxAndStoredStateOfCommenting: function ($field) {
                var result = $.Deferred();
                JIRA.SmartAjax.makeRequest({
                    type: 'GET',
                    async: false,
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/settings/email-to-comment',
                    data: {
                        projectKey: this.getProjectKey
                    },
                    success: function (data) {
                        $field.attr('checked', data === 'true');
                        result.resolve(this);
                    }
                });
                return result;
            }
        });

        return SADialogEmail;
    });