define('smart-attachments/analytics-transfer', [
    'jquery',
    'underscore',
    'smart-attachments/analytics-data'
], function ($, _, Data) {

    var APP_NAME = "Smart Attachments";
    var PROJECT_SPECIFIC = "Project specific";
    var SHARED = "Shared";
    var pluginVersion, pluginLicenseSize, pluginLicenseType, serverHash;

    function event(screenName, eventCategory, eventAction, dimensions) {
        var sid = getServerHash();
        var uid = getUserId();
        var appVersion = getPluginVersion();

        var data = {
            hitType: 'event',
            eventCategory: eventCategory,
            eventAction: eventAction,
            screenName: screenName,
            anonymizeIp: true,
            appVersion: appVersion,
            appName: APP_NAME,
            userId: sid,
            eventLabel: sid
        };
        data[Data.dimensions.SERVER_ID_HASH] = sid;
        data[Data.dimensions.USER_ID] = uid;
        data[Data.dimensions.APP_VERSION] = appVersion;
        data = $.extend({}, data, dimensions);

        saga('send', data);
    }

    function getPluginVersion() {
        if (!pluginVersion) {
            pluginVersion = $("meta[name='sa-plugin-version']").attr('content');
        }
        return pluginVersion;
    }

    function getPluginLicenseSize() {
        if (!pluginLicenseSize) {
            pluginLicenseSize = $("meta[name='sa-lic-size']").attr('content')
        }
        return pluginLicenseSize;
    }

    function getServerHash() {
        if (!serverHash) {
            serverHash = $("meta[name='sa-server-hash']").attr("content");
        }
        return serverHash;
    }

    function getJiraLicenseType() {
        if (!pluginLicenseType) {
            pluginLicenseType = $("meta[name='sa-lic-type']").attr("content");
        }
        return pluginLicenseType;
    }

    function getUserId() {
        var storageKey = 'com.stiltsoft.jira.attachcategory.analytics.userId';
        var id = localStorage.getItem(storageKey);
        if (!id) {
            id = getRandomInt();
            localStorage.setItem(storageKey, id.toString());
        }
        return id;
    }

    function getProjectType() {
        return $('meta[name=projectType]').attr("content");
    }

    function getProjectId() {
        return $('meta[name=projectId]').attr("content");
    }

    function getDeploymentType() {
        return $("meta[name='sa-deployment']").attr("content");
    }

    function getRandomInt() {
        var max = 99999999999999999;
        return Math.floor(Math.random() * (max - 1));
    }

    function parseFileExtension(fileName) {
        if (!fileName) {
            return "no file format";
        }

        if (fileName.length) {
            fileName = fileName.toLowerCase();
        }
        var indexDot = fileName.lastIndexOf(".");
        var length = fileName.length;
        var diff = length - indexDot - 1;
        return (diff !== length) && (diff <= 5) ? fileName.substring(indexDot + 1) : "no file format";
    }

    function makeSchemeId(schemeId) {
        return schemeId + '_' + getServerHash();
    }

    function makeCategoryId(categoryId) {
        return categoryId + '_' + getServerHash();
    }

    function getCategoryIssueTypeRatio(scheme) {
        var total = scheme.categories.length;
        var categoryWithIssueTypes = 0;
        scheme.categories.forEach(function (category) {
            if (category.issueTypes.length) {
                categoryWithIssueTypes++;
            }
        });
        return categoryWithIssueTypes + '/' + total;
    }

    function getCategoryAccessRestrictionRatio(scheme) {
        var total = scheme.categories.length;
        var categoryWithAccessRestrictions = 0;
        scheme.categories.forEach(function (category) {
            var accessRestrictions = category.accessRestrictions;
            if (accessRestrictions.users.length || accessRestrictions.groups.length
                || accessRestrictions.projectRoles.length || accessRestrictions.fields.length) {
                categoryWithAccessRestrictions++;
            }
        });
        return categoryWithAccessRestrictions + '/' + total;
    }

    function getCategoryAutoDistributionRatio(scheme) {
        var total = scheme.categories.length;
        var categoryWithAutoDistribution = 0;
        scheme.categories.forEach(function (category) {
            if (category.autoDistribution && category.autoDistribution.value) {
                categoryWithAutoDistribution++;
            }
        });
        return categoryWithAutoDistribution + '/' + total;
    }

    function getCommonSchemeDimensions(scheme) {
        var dimensions = getBaseDimensions();
        dimensions[Data.dimensions.SCHEME_ID] = makeSchemeId(scheme.id);
        dimensions[Data.dimensions.CATEGORY_COUNT] = scheme.categories.length.toString();
        dimensions[Data.dimensions.ISSUE_TYPES_RATIO] = getCategoryIssueTypeRatio(scheme);
        dimensions[Data.dimensions.ACCESS_RESTRICTIONS_RATIO] = getCategoryAccessRestrictionRatio(scheme);
        dimensions[Data.dimensions.AUTO_DISTRIBUTION_RULE_RATIO] = getCategoryAutoDistributionRatio(scheme);
        return dimensions;
    }

    function getCommonSchemeAndProjectDimensions(scheme) {
        var dimensions = getCommonSchemeDimensions(scheme);
        dimensions[Data.dimensions.PROJECT_TYPE] = getProjectType();
        dimensions[Data.dimensions.PROJECT_ID] = getProjectId();
        return dimensions;
    }

    function collectCategoryEventsFromNewScheme(screen, analyticsCategory, scheme, dimensionInjector) {
        scheme.categories.forEach(function (schemeCategory) {
            var dimensions = getCommonCategoryDimensions(scheme.id, schemeCategory.id);
            if (dimensionInjector) {
                dimensionInjector(dimensions);
            }
            triggerCategoryCreatedEvents(schemeCategory, screen, analyticsCategory, dimensions);
        });
    }

    function checkCategoryModificationsAndTriggerEvents(originalCategory, editedCategory, screen, analyticsCategory, dimensions) {
        if (isCategoryRenamed(originalCategory, editedCategory)) {
            event(screen, analyticsCategory, Data.actions.CATEGORY_RENAMED, dimensions);
        }
        if (isCategoryIssueTypesChanged(originalCategory, editedCategory)) {
            event(screen, analyticsCategory, Data.actions.CATEGORY_ISSUE_TYPES_APPLIED, dimensions);
        }
        if (isCategoryAccessRestrictionsChanged(originalCategory, editedCategory)) {
            event(screen, analyticsCategory, Data.actions.CATEGORY_ACCESS_RESTRICTIONS_APPLIED, dimensions);
        }
        if (isCategoryAutoDistributionChanged(originalCategory, editedCategory)) {
            event(screen, analyticsCategory, Data.actions.CATEGORY_AUTODISTRIBUTION_RULES_APPLIED, dimensions);
        }
    }

    function isCategoryRenamed(originalCategory, editedCategory) {
        return originalCategory.name !== editedCategory.name;
    }

    function isCategoryAccessRestrictionsChanged(originalCategory, editedCategory) {
        if (isCategoryAccessRestrictionGroupsChanged(originalCategory, editedCategory)) {
            return true;
        }
        if (isCategoryAccessRestrictionUsersChanged(originalCategory, editedCategory)) {
            return true;
        }
        if (isCategoryAccessRestrictionProjectRolesChanged(originalCategory, editedCategory)) {
            return true;
        }
        return !!isCategoryAccessRestrictionCustomFieldsChanged(originalCategory, editedCategory);
    }

    function isCategoryIssueTypesChanged(originalCategory, editedCategory) {
        var originalIssueTypeIds = pickFieldValues(originalCategory.issueTypes, 'id');
        var editedIssueTypeIds = pickFieldValues(editedCategory.issueTypes, 'id');
        return !_.isEqual(originalIssueTypeIds.sort(), editedIssueTypeIds.sort());
    }

    function isCategoryAccessRestrictionUsersChanged(originalCategory, editedCategory) {
        var originalUserKeys = pickFieldValues(originalCategory.accessRestrictions.users, 'key');
        var editedUserKeys = pickFieldValues(editedCategory.accessRestrictions.users, 'key');
        return !_.isEqual(originalUserKeys.sort(), editedUserKeys.sort());
    }

    function isCategoryAccessRestrictionGroupsChanged(originalCategory, editedCategory) {
        var originalGroups = pickFieldValues(originalCategory.accessRestrictions.groups, 'name');
        var editedGroups = pickFieldValues(editedCategory.accessRestrictions.groups, 'name');
        return !_.isEqual(originalGroups.sort(), editedGroups.sort());
    }

    function isCategoryAccessRestrictionProjectRolesChanged(originalCategory, editedCategory) {
        var originalProjectRoleIds = pickFieldValues(originalCategory.accessRestrictions.projectRoles, 'id');
        var editedProjectRoleIds = pickFieldValues(editedCategory.accessRestrictions.projectRoles, 'id');
        return !_.isEqual(originalProjectRoleIds.sort(), editedProjectRoleIds.sort());
    }

    function isCategoryAccessRestrictionCustomFieldsChanged(originalCategory, editedCategory) {
        var originalCustomFieldIds = pickFieldValues(originalCategory.accessRestrictions.fields, 'id');
        var editedCustomFieldIds = pickFieldValues(editedCategory.accessRestrictions.fields, 'id');
        return !_.isEqual(originalCustomFieldIds.sort(), editedCustomFieldIds.sort());
    }

    function isCategoryAutoDistributionChanged(originalCategory, editedCategory) {
        return !_.isEqual(originalCategory.autoDistribution, editedCategory.autoDistribution);
    }

    function collectCategoryEventsFromEditedScheme(screen, analyticsCategory, originalScheme, editedScheme, dimensionInjector) {
        editedScheme.categories.forEach(function (editedCategory) {
            var dimensions = getCommonCategoryDimensions(editedScheme.id, editedCategory.id);
            if (dimensionInjector) {
                dimensionInjector(dimensions);
            }
            var originalCategory = getCategoryWithId(originalScheme.categories, editedCategory.id);

            if (originalCategory) {
                checkCategoryModificationsAndTriggerEvents(originalCategory, editedCategory, screen, analyticsCategory, dimensions);
            } else {
                triggerCategoryCreatedEvents(editedCategory, screen, analyticsCategory, dimensions);
            }
        });

        originalScheme.categories.forEach(function (originalCategory) {
            var editedCategory = getCategoryWithId(editedScheme.categories, originalCategory.id);
            if (!editedCategory) {
                var dimensions = getCommonCategoryDimensions(editedScheme.id, originalCategory.id);
                event(screen, analyticsCategory, Data.actions.CATEGORY_DELETED, dimensions);
            }
        });
    }

    function triggerCategoryCreatedEvents(schemeCategory, screen, analyticsCategory, dimensions) {
        event(screen, analyticsCategory, Data.actions.CATEGORY_ADDED, dimensions);
        if (schemeCategory.issueTypes.length) {
            event(screen, analyticsCategory, Data.actions.CATEGORY_ISSUE_TYPES_APPLIED, dimensions);
        }
        if (schemeCategory.accessRestrictions.groups.length || schemeCategory.accessRestrictions.users.length ||
            schemeCategory.accessRestrictions.projectRoles.length || schemeCategory.accessRestrictions.fields.length) {
            event(screen, analyticsCategory, Data.actions.CATEGORY_ACCESS_RESTRICTIONS_APPLIED, dimensions);
        }
        if (schemeCategory.autoDistribution && schemeCategory.autoDistribution.value) {
            event(screen, analyticsCategory, Data.actions.CATEGORY_AUTODISTRIBUTION_RULES_APPLIED, dimensions);
        }
    }

    function getCommonCategoryDimensions(schemeId, categoryId) {
        var dimensions = getBaseDimensions();
        dimensions[Data.dimensions.SCHEME_ID] = makeSchemeId(schemeId);
        dimensions[Data.dimensions.CATEGORY_ID] = makeCategoryId(categoryId);
        return dimensions;
    }

    function getCategoryAttachmentMethodDimensions(categoryId, attachmentsCount) {
        var dimensions = getBaseDimensions();
        dimensions[Data.dimensions.ATTACHMENTS_COUNT] = attachmentsCount;
        dimensions[Data.dimensions.CATEGORY_ID] = makeCategoryId(categoryId);
        return dimensions;
    }

    function getAttachmentDimensions(attachmentName) {
        var dimensions = getBaseDimensions();
        dimensions[Data.dimensions.FILE_EXTENSION] = parseFileExtension(attachmentName);
        return dimensions;
    }

    function getCommentDimensions(props) {
        var dimensions = getBaseDimensions();
        var attachmentName = props[0];
        dimensions[Data.dimensions.FILE_EXTENSION] = parseFileExtension(attachmentName);
        return dimensions;
    }

    function getBulkOperationDimensions(attachmentsCount) {
        var dimensions = getBaseDimensions();
        dimensions[Data.dimensions.ATTACHMENTS_COUNT] = attachmentsCount;
        return dimensions;
    }

    function getFormDimensions(categoryId, attachmentsCount) {
        var dimensions = getBaseDimensions();
        dimensions[Data.dimensions.CATEGORY_ID] = categoryId;
        dimensions[Data.dimensions.ATTACHMENTS_COUNT] = attachmentsCount;
        return dimensions;
    }

    function getBaseDimensions() {
        var dimensions = {};
        dimensions[Data.dimensions.SERVER_ID_HASH] = getServerHash();
        dimensions[Data.dimensions.JIRA_LICENSE_TYPE] = getJiraLicenseType();
        dimensions[Data.dimensions.PLUGIN_LICENSE_SIZE] = getPluginLicenseSize();
        dimensions[Data.dimensions.USER_ID] = getUserId();
        dimensions[Data.dimensions.DEPLOYMENT] = getDeploymentType();
        return dimensions;
    }

    function getCategoryWithId(categories, categoryId) {
        for (var i = 0; i < categories.length; i++) {
            var category = categories[i];
            if (categoryId === category.id) {
                return category;
            }
        }
    }

    function pickFieldValues(items, fieldName) {
        var values = [];
        for (var i = 0; i < items.length; i++) {
            values.push(items[i][fieldName]);
        }
        return values;
    }

    function eventDnDAttachment(props, eventAction) {
        var categoryId = props[0];
        var attachmentsCount = props[1];
        var dimensions = getCategoryAttachmentMethodDimensions(categoryId, attachmentsCount);
        event(Data.screens.ISSUE, Data.categories.ATTACHMENT_METHOD, eventAction, dimensions);
    }

    function reduceAttachmentsCount(categories) {
        return _.reduce(categories, function (memo, category) {
            var documentAttachmentsCount = reduceDocumentAttachmentsCount(category.documents);
            return memo + category.attachmentIds.length + documentAttachmentsCount;
        }, 0);
    }

    function reduceDocumentAttachmentsCount(documents) {
        return _.reduce(documents, function (memo, document) {
            return memo + document.attachmentIds.length;
        }, 0);
    }

    function isAttachmentsCountCollected(issueId) {
        var storageKey = 'com.stiltsoft.jira.attachcategory.analytics.collectedIssueIds';
        var issueIds = localStorage.getItem(storageKey);

        if (issueIds) {
            issueIds = JSON.parse(issueIds);
            if (issueIds.indexOf(issueId) !== -1) {
                return true;
            }
        } else {
            issueIds = [];
        }

        issueIds.push(issueId);
        localStorage.setItem(storageKey, JSON.stringify(issueIds));
        return false;
    }

    function getSchemeType(scheme) {
        return scheme.jiraProjectIds.length === 1 ? PROJECT_SPECIFIC : SHARED;
    }

    function attachmentUploadSingleFile(fileName) {
        var dimensions = getAttachmentDimensions(fileName);
        event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.ATTACHMENT_UPLOAD, dimensions);
    }

    var analytics = {

        attachmentUpload: function (props) {
            var tempAttachments = props[0];
            if (_.isArray(tempAttachments)) {
                tempAttachments.forEach(function (tempAttachment) {
                    attachmentUploadSingleFile(tempAttachment.name);
                });
            } else {
                attachmentUploadSingleFile(tempAttachments.name);
            }
        },

        attachmentToEmailSent: function (props) {
            var fileName = props[0];
            var dimensions = getAttachmentDimensions(fileName);
            event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.ATTACHMENT_TO_EMAIL_SENT, dimensions);
        },

        attachmentToCategoryMoved: function (props) {
            var fileName = props[0];
            var dimensions = getAttachmentDimensions(fileName);
            event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.ATTACHMENT_TO_CATEGORY_MOVED, dimensions);
        },

        disableAnalytics: function (props) {
            var screen = props[0];
            var dimensions = getBaseDimensions();
            event(screen, Data.categories.EVENT_ANALYTICS, Data.actions.ANALYTICS_OFF, dimensions);
        },

        enableAnalytics: function (props) {
            var screen = props[0];
            var dimensions = getBaseDimensions();
            event(screen, Data.categories.EVENT_ANALYTICS, Data.actions.ANALYTICS_ON, dimensions);
        },

        workflowConditionCategoryNotEmptyAdded: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_CONDITION, Data.actions.CATEGORY_NOT_EMPTY_ADD, getBaseDimensions());
        },

        workflowValidatorCategoryNotEmptyAdded: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_VALIDATOR, Data.actions.CATEGORY_NOT_EMPTY_ADD, getBaseDimensions());
        },

        workflowConditionFileUploaderUserAdded: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_CONDITION, Data.actions.FILE_UPL_USER_ADD, getBaseDimensions());
        },

        workflowValidatorFileUploaderUserAdded: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_VALIDATOR, Data.actions.FILE_UPL_USER_ADD, getBaseDimensions());
        },

        workflowConditionFileUploaderGroupAdded: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_CONDITION, Data.actions.FILE_UPL_GROUP_ADD, getBaseDimensions());
        },

        workflowValidatorFileUploaderGroupAdded: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_VALIDATOR, Data.actions.FILE_UPL_GROUP_ADD, getBaseDimensions());
        },

        workflowConditionFileUploaderProjectRoleAdded: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_CONDITION, Data.actions.FILE_UPL_PROJECT_ROLE_ADD, getBaseDimensions());
        },

        workflowValidatorFileUploaderProjectRoleAdded: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_VALIDATOR, Data.actions.FILE_UPL_PROJECT_ROLE_ADD, getBaseDimensions());
        },

        workflowPostFunctionAttachmentFromCategoryRemoval: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_POST_FUNCTION, Data.actions.ATTACH_FROM_CATEGORY_REMOVE, getBaseDimensions());
        },

        workflowPostFunctionAttachmentToCategoryMoving: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_POST_FUNCTION, Data.actions.ATTACH_TO_CATEGORY_MOVE, getBaseDimensions());
        },

        workflowPostFunctionAttachmentToEmailSending: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_POST_FUNCTION, Data.actions.SEND_TO_EMAIL, getBaseDimensions());
        },

        workflowConditionLimitAttachmentsInCategoryAdded: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_CONDITION, Data.actions.LIMIT_ATT_IN_CATEGORY, getBaseDimensions());
        },

        workflowValidatorLimitAttachmentsInCategoryAdded: function () {
            event(Data.screens.WORKFLOW, Data.categories.WORKFLOW_VALIDATOR, Data.actions.LIMIT_ATT_IN_CATEGORY, getBaseDimensions());
        },

        schemeCreatedFromProjectAdministration: function (props) {
            var createdScheme = props[0];
            var dimensions = getCommonSchemeAndProjectDimensions(createdScheme);
            dimensions[Data.dimensions.SCHEME_TYPE] = PROJECT_SPECIFIC;
            event(Data.screens.PROJECT_SCHEME_ADMINISTRATION, Data.categories.PROJECT_SCHEME, Data.actions.SCHEME_CREATED, dimensions);
            collectCategoryEventsFromNewScheme(Data.screens.PROJECT_SCHEME_ADMINISTRATION, Data.categories.PROJECT_SCHEME, createdScheme, function (categoryDimensions) {
                categoryDimensions[Data.dimensions.PROJECT_TYPE] = getProjectType();
                categoryDimensions[Data.dimensions.PROJECT_ID] = getProjectId();
                categoryDimensions[Data.dimensions.SCHEME_TYPE] = PROJECT_SPECIFIC;
            });
        },

        schemeCreatedFromGlobalAdministration: function (props) {
            var createdScheme = props[0];
            var dimensions = getCommonSchemeDimensions(createdScheme);
            event(Data.screens.GLOBAL_SCHEME_ADMINISTRATION, Data.categories.GLOBAL_SCHEME, Data.actions.SCHEME_CREATED, dimensions);
            collectCategoryEventsFromNewScheme(Data.screens.GLOBAL_SCHEME_ADMINISTRATION, Data.categories.GLOBAL_SCHEME, createdScheme);
        },

        schemeEditedFromProjectAdministration: function (props) {
            var originalScheme = props[0];
            var editedScheme = props[1];
            var dimensions = getCommonSchemeAndProjectDimensions(editedScheme);
            dimensions[Data.dimensions.SCHEME_TYPE] = PROJECT_SPECIFIC;
            event(Data.screens.PROJECT_SCHEME_ADMINISTRATION, Data.categories.PROJECT_SCHEME, Data.actions.SCHEME_EDITED, dimensions);
            collectCategoryEventsFromEditedScheme(Data.screens.PROJECT_SCHEME_ADMINISTRATION, Data.categories.PROJECT_SCHEME, originalScheme, editedScheme, function (categoryDimensions) {
                categoryDimensions[Data.dimensions.PROJECT_TYPE] = getProjectType();
                categoryDimensions[Data.dimensions.PROJECT_ID] = getProjectId();
            });
        },

        schemeEditedFromGlobalAdministration: function (props) {
            var originalScheme = props[0];
            var editedScheme = props[1];
            var dimensions = getCommonSchemeDimensions(editedScheme);
            event(Data.screens.GLOBAL_SCHEME_ADMINISTRATION, Data.categories.GLOBAL_SCHEME, Data.actions.SCHEME_EDITED, dimensions);
            collectCategoryEventsFromEditedScheme(Data.screens.GLOBAL_SCHEME_ADMINISTRATION, Data.categories.GLOBAL_SCHEME, originalScheme, editedScheme);
        },

        schemeDeleted: function (props) {
            var deletedScheme = props[0];
            var dimensions = getCommonSchemeDimensions(deletedScheme);
            event(Data.screens.GLOBAL_SCHEME_ADMINISTRATION, Data.categories.GLOBAL_SCHEME, Data.actions.SCHEME_DELETED, dimensions);
        },

        schemeCopied: function (props) {
            var deletedScheme = props[0];
            var dimensions = getCommonSchemeDimensions(deletedScheme);
            event(Data.screens.GLOBAL_SCHEME_ADMINISTRATION, Data.categories.GLOBAL_SCHEME, Data.actions.SCHEME_COPIED, dimensions);
        },

        schemeAssociatedFromProjectAdministration: function (props) {
            var scheme = props[0];
            var dimensions = getCommonSchemeAndProjectDimensions(scheme);
            dimensions[Data.dimensions.SCHEME_TYPE] = getSchemeType(scheme);
            event(Data.screens.PROJECT_SCHEME_ADMINISTRATION, Data.categories.PROJECT_SCHEME, Data.actions.SCHEME_ASSOCIATED, dimensions);
        },

        schemeSwitchedFromProjectAdministration: function (props) {
            var scheme = props[0];
            var dimensions = getCommonSchemeAndProjectDimensions(scheme);
            dimensions[Data.dimensions.SCHEME_TYPE] = getSchemeType(scheme);
            event(Data.screens.PROJECT_SCHEME_ADMINISTRATION, Data.categories.PROJECT_SCHEME, Data.actions.SCHEME_SWITCHED, dimensions);
        },

        schemeAssociatedFromGlobalAdministration: function (props) {
            var scheme = props.scheme;
            var projectTypeById = props.projectTypeById;

            if ($.isEmptyObject(projectTypeById)) {
                return;
            }

            var dimensions = getCommonSchemeDimensions(scheme);
            dimensions[Data.dimensions.SCHEME_TYPE] = getSchemeType(scheme);

            props.associatedProjectIds.forEach(function (associatedProjectId) {
                var projectType = projectTypeById[associatedProjectId];
                var dimensionsCopy = _.clone(dimensions);
                dimensionsCopy[Data.dimensions.PROJECT_TYPE] = projectType;
                dimensionsCopy[Data.dimensions.PROJECT_ID] = associatedProjectId;
                event(Data.screens.GLOBAL_SCHEME_ADMINISTRATION, Data.categories.GLOBAL_SCHEME, Data.actions.SCHEME_ASSOCIATED, dimensionsCopy);
            });

            props.disassociatedProjectIds.forEach(function (disassociatedProjectId) {
                var projectType = projectTypeById[disassociatedProjectId];
                var dimensionsCopy = _.clone(dimensions);
                dimensionsCopy[Data.dimensions.PROJECT_TYPE] = projectType;
                dimensionsCopy[Data.dimensions.PROJECT_ID] = disassociatedProjectId;
                event(Data.screens.GLOBAL_SCHEME_ADMINISTRATION, Data.categories.GLOBAL_SCHEME, Data.actions.SCHEME_DISASSOCIATED, dimensionsCopy);
            });
        },

        attachmentDnDToIssue: function (props) {
            eventDnDAttachment(props, Data.actions.ATTACHMENT_DRAG_N_DROP_TO_ISSUE);
        },

        agileBoardDnDAttachment: function (props) {
            eventDnDAttachment(props, Data.actions.AGILE_BOARD_DRAG_N_DROP_ATTACHMENT);
        },

        issueNavigatorDnDAttachment: function (props) {
            eventDnDAttachment(props, Data.actions.ISSUE_NAVIGATOR_DRAG_N_DROP_ATTACHMENT);
        },

        searchForIssuesAttachmentDnD: function (props){
            eventDnDAttachment(props, Data.actions.SEARCH_FOR_ISSUES_ATTACHMENT_DRAG_N_DROP);
        },

        queuesDnDAttachment: function (props) {
            eventDnDAttachment(props, Data.actions.QUEUES_DRAG_N_DROP_ATTACHMENT);
        },

        attachmentRenamed: function (props) {
            var name = props[0];
            var dimensions = getAttachmentDimensions(name);
            event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.ATTACHMENT_RENAMED, dimensions);
        },

        attachmentWithGoEditEdited: function (props) {
            var name = props[0];
            var dimensions = getAttachmentDimensions(name);
            event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.ATTACHMENT_WITH_GOEDIT_EDITED, dimensions);
        },

        bulkOperationAttachmentsDeleted: function (props) {
            var attachmentsCount = props[0];
            var dimensions = getBulkOperationDimensions(attachmentsCount);
            event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.BULK_OPERATION_ATTACHMENTS_DELETED, dimensions);
        },

        bulkOperationAttachmentsDownloaded: function (props) {
            var attachmentsCount = props[0];
            var dimensions = getBulkOperationDimensions(attachmentsCount);
            event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.BULK_OPERATION_ATTACHMENTS_DOWNLOADED, dimensions);
        },

        bulkOperationAttachmentsToCategoryMoved: function (props) {
            var attachmentsCount = props[0];
            var dimensions = getBulkOperationDimensions(attachmentsCount);
            event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.BULK_OPERATION_ATTACHMENTS_TO_CATEGORY_MOVED, dimensions);
        },

        bulkOperationAttachmentsToEmailSent: function (props) {
            var attachmentsCount = props[0];
            var dimensions = getBulkOperationDimensions(attachmentsCount);
            event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.BULK_OPERATION_ATTACHMENTS_TO_EMAIL_SENT, dimensions);
        },

        bulkOperationAttachmentsArchived: function (props) {
            var attachmentsCount = props[0];
            var dimensions = getBulkOperationDimensions(attachmentsCount);
            event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.BULK_OPERATION_ATTACHMENTS_ARCHIVED, dimensions);
        },

        commentThreadAdded: function (props) {
            event(Data.screens.ISSUE, Data.categories.COMMENT_THREADS, Data.actions.COMMENT_THREAD_ADDED, getCommentDimensions(props));
        },

        commentThreadDeleted: function (props) {
            event(Data.screens.ISSUE, Data.categories.COMMENT_THREADS, Data.actions.COMMENT_THREAD_DELETED, getCommentDimensions(props));
        },

        newCommentAdded: function (props) {
            event(Data.screens.ISSUE, Data.categories.COMMENT_THREADS, Data.actions.NEW_COMMENT_ADDED, getCommentDimensions(props));
        },

        commentDeleted: function (props) {
            event(Data.screens.ISSUE, Data.categories.COMMENT_THREADS, Data.actions.COMMENT_DELETED, getCommentDimensions(props));
        },

        commentEdited: function (props) {
            event(Data.screens.ISSUE, Data.categories.COMMENT_THREADS, Data.actions.COMMENT_EDITED, getCommentDimensions(props));
        },

        createIssueFormDnDAttachment: function (props) {
            var dimensions = getFormDimensions(props[0], props[1]);
            event(Data.screens.OPENING_FORMS, Data.categories.ATTACHMENT_METHOD, Data.actions.CREATE_ISSUE_FORM_DRAG_N_DROP_ATTACHMENT, dimensions);
        },

        createIssueFormAttachmentBrowseUpload: function (props) {
            var dimensions = getFormDimensions(props[0], props[1]);
            event(Data.screens.OPENING_FORMS, Data.categories.ATTACHMENT_METHOD, Data.actions.CREATE_ISSUE_FORM_ATTACHMENT_BROWSE_UPLOAD, dimensions);
        },

        attachFilesFormDNDAttachment: function (props) {
            var dimensions = getFormDimensions(props[0], props[1]);
            event(Data.screens.OPENING_FORMS, Data.categories.ATTACHMENT_METHOD, Data.actions.ATTACH_FILES_FORM_DRAG_N_DROP_ATTACHMENT, dimensions);
        },

        attachFilesFormAttachmentBrowseUpload: function (props) {
            var dimensions = getFormDimensions(props[0], props[1]);
            event(Data.screens.OPENING_FORMS, Data.categories.ATTACHMENT_METHOD, Data.actions.ATTACH_FILES_FORM_ATTACHMENT_BROWSE_UPLOAD, dimensions);
        },

        attachScreenshotFormAttachment: function (props) {
            var dimensions = getFormDimensions(props[0], props[1]);
            event(Data.screens.OPENING_FORMS, Data.categories.ATTACHMENT_METHOD, Data.actions.ATTACH_SCREENSHOT_FORM_ATTACHMENT, dimensions);
        },

        transitionFormDNDropAttachment: function (props) {
            var dimensions = getFormDimensions(props[0], props[1]);
            event(Data.screens.OPENING_FORMS, Data.categories.ATTACHMENT_METHOD, Data.actions.TRANSITION_FORM_DRAG_N_DROP_ATTACHMENT, dimensions);
        },

        transitionFormAttachmentBrowseUpload: function (props) {
            var dimensions = getFormDimensions(props[0], props[1]);
            event(Data.screens.OPENING_FORMS, Data.categories.ATTACHMENT_METHOD, Data.actions.TRANSITION_FORM_ATTACHMENT_BROWSE_UPLOAD, dimensions);
        },

        countAttachmentsInIssue: function (props) {
            var issueData = props[0];
            var issueId = issueData.issueId;

            if (isAttachmentsCountCollected(issueId)) {
                return;
            }

            var attachmentsCount = reduceAttachmentsCount(issueData.categories);
            var dimensions = getBulkOperationDimensions(attachmentsCount);
            event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.COUNT_ATTACHMENTS_IN_ISSUE, dimensions);
        },

        manualDocumentRevisionUpload: function () {
            var dimensions = getBaseDimensions();
            event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.MANUAL_DOCUMENT_REVISION_UPLOAD, dimensions);
        },

        automaticDocumentRevisionUpload: function (props) {
            var tempAttachments = props[0];
            var dimensions = getBaseDimensions();

            tempAttachments.forEach(function (tempAttachment) {
                if (tempAttachment.parentId) {
                    dimensions[Data.dimensions.FILE_EXTENSION] = parseFileExtension(tempAttachment.name);
                    event(Data.screens.ISSUE, Data.categories.ATTACHMENT, Data.actions.AUTOMATIC_DOCUMENT_REVISION_UPLOAD, dimensions);
                }
            });
        }
    };

    return analytics;
});