define('smart-attachments/extension/CategoryDataManager',
    [
        'jquery',
        'underscore',
        'jira/lib/class',
        'jira/util/events',
        'jira/util/events/types'
    ],
    function ($, _, Class, Events, Types) {
        var isEqualIssueKeyFunction = function (issueKey) {
            return function (watcher) {
                return watcher.issueKey === issueKey;
            }
        };

        var not = function (predicate) {
            return function () {
                return !predicate.apply(this, arguments);
            };
        };

        var activeRequests = {};
        var data = [];
        var watchers = [];

        if (!window.CategoryDataManager) {
            var CategoryDataManager = Class.extend({
                init: function () {
                    Events.bind('change:attachments', this.changeHandler.bind(this));
                },

                changeHandler: function (event, Issue) {
                    var issueKey = Issue.getIssueKey();
                    if (data[issueKey]) {
                        console.log('Reset categories for ' + issueKey);
                        delete data[issueKey];
                    }
                },

                fetchData: function (issueKey) {
                    console.log('Start fetch ' + issueKey);
                    var deferred = $.Deferred();
                    JIRA.SmartAjax.makeRequest({
                        type: 'GET',
                        url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments',
                        data: {
                            issueKey: issueKey,
                            extraAttributes: true
                        },
                        success: function (data, statusText, xhr) {
                            if (xhr.status === 204) {
                                return;
                            }
                            deferred.resolve(data);
                        },
                        error: function () {
                            console.log('Probably Smart Attachments disabled');
                            deferred.reject({});
                        }
                    });
                    return deferred;
                },

                onDataReceive: function (issueKey) {
                    return function (data) {
                        data[issueKey] = data;
                        delete activeRequests[issueKey];
                        var isEqualIssueKey = isEqualIssueKeyFunction(issueKey);
                        var watcher = _.find(watchers, isEqualIssueKey);
                        if (watcher) {
                            watcher.onFetch(data);
                        }
                    }
                },

                makeRequest: function(issueKey) {
                    if (!activeRequests[issueKey]) {
                        console.log("Send request for " + issueKey);
                        var fetch = this.fetchData(issueKey);
                        activeRequests[issueKey] = fetch;
                        fetch.then(this.onDataReceive(issueKey).bind(this));
                        fetch.fail(function () {
                            Events.unbind(Types.NEW_CONTENT_ADDED + '.smart');
                        });
                    } else {
                        console.log('Request already sent for ' + issueKey);
                    }
                },

                register: function (newWatcher) {
                    var issueKey = newWatcher.issueKey;
                    var isEqualIssueKey = isEqualIssueKeyFunction(issueKey);
                    var logId = newWatcher.getLogId();
                    var oldWatcher = _.find(watchers, isEqualIssueKey);
                    if (oldWatcher) {
                        if (oldWatcher.isAttached() && oldWatcher.isSameContext(newWatcher)) {
                            console.log('Ignore watcher ' + logId);
                            return;
                        }
                        oldWatcher.disconnectContainer();
                        watchers = _.filter(watchers, not(isEqualIssueKey))
                    }

                    watchers.push(newWatcher);

                    if (!data[issueKey]) {
                        this.makeRequest(issueKey);
                    }

                    var dataExists = data[issueKey];
                    if (dataExists) {
                        newWatcher.onFetch(dataExists);
                    }
                }
            });

            window.CategoryDataManager = new CategoryDataManager();
        }

        return window.CategoryDataManager;
    });

define('smart-attachments/extension/CategoryPanelWatcher', [
    'jquery',
    'jira/util/data/meta',
    'jira/util/events',
    'jira/util/events/types',
    'smart-attachments/AttachmentPanel'
], function ($, Meta, Events, Types, AttachCat) {

    function CategoryPanelWatcher() {
        this.init.apply(this, arguments);
    }

    var id = 0;

    _.extend(CategoryPanelWatcher.prototype, {
        init: function (element, issue, isAgile) {
            var issueKey = issue.getIssueKey();
            this.id = ++id;
            this.element = element;
            this.issueKey = issueKey;
            this.issue = issue;
            this.isAgile = isAgile;
            this.connected = false;
            //Events.bind(Types.PANEL_REFRESHED, this.panelRefreshed.bind(this));
            console.log('Init watcher ' + this.getLogId());
        },

        getLogId: function () {
            return this.issueKey + ':' + this.id
        },

        connectContainer: function (pluginConfig) {
            console.log("Connect to container: " + this.getLogId());
            this.connected = true;
            var $node = $(this.element);
            var attachCat = new AttachCat($node, this.issue, this.isAgile);
            attachCat.attachListeners(pluginConfig);
            this.attachCat = attachCat;
        },

       /* panelRefreshed: function (event, panel, $new, $existing) {
            if (!$existing.find(this.$node).is('*')) {
                return;
            }
            console.log("panelRefreshed " + this.getLogId());
        },*/

        onFetch: function (data) {
            console.log("Data ready " + this.getLogId());
            if (!this.connected) {
                if (this.isAttached()) {
                    this.connectContainer(data);
                } else {
                    console.log("Element not attached " + this.issueKey);
                }
            } else {
                console.log("Already connected " + this.getLogId());
            }
        },

        isAttached: function () {
            return $.contains(document, this.element);
        },

        isSameContext: function (watcher) {
            return this.element.isEqualNode(watcher.element);
        },

        onContentAdded: function () {
            console.log("On content added: " + this.getLogId());
            if (!this.isAttached()) {
                this.disconnectContainer();
            }
        },

        disconnectContainer: function () {
            console.log('disconnectContainer ' + this.getLogId());
            if (this.attachCat) {
                this.attachCat.destroy();
                delete this.attachCat;
            }
            this.connected = false;
        }
    });
    return CategoryPanelWatcher;
});

require([
    'jquery',
    'jira/util/data/meta',
    'smart-attachments/service-desk-integration/CommentDNDExecutor',
    'smart-attachments/extension/CategoryDataManager',
    'smart-attachments/extension/CategoryPanelWatcher'
], function ($, Meta, ExecutorInstaller, CategoryDataManager, CategoryPanelWatcher) {

    function getPageIssue($el) {
        var keyValue = $el.closest('#issue-content').find('#key-val');
        return {issueKey: keyValue.data('issue-key'), issueId: keyValue.attr('rel')};
    }

    function getAgileIssue($el) {
        var keyValue = $el.closest('#ghx-detail-view').find('#ghx-detail-issue');
        return {issueKey: keyValue.data('issuekey'), issueId: keyValue.data('issueid')};
    }

    var Issue = function ($element, isAgile) {
        var jiraIssue = require('jira/issue');
        var issueKey = jiraIssue.getIssueKey();
        var issueId = jiraIssue.getIssueId();
        if (!issueKey) {
            var pageIssue = isAgile ? getAgileIssue($element) : getPageIssue($element);
            issueKey = pageIssue.issueKey;
            issueId = pageIssue.issueId;
        }
        return {
            getIssueKey: function () {
                return issueKey;
            },
            getIssueId: function () {
                return issueId;
            }
        }
    };

    var DEFAULT_TARGET_ID = 'attachment-cat-issue-section';

    function createCategoryPanelWatcher($context) {
        var isAgile = false;
        var targetId = DEFAULT_TARGET_ID;
        if (document.querySelector('#gh')) {
            targetId = 'ghx-tab-com-stiltsoft-jira-smart-attachments-attachment-cat-issue-section-agile';
            $context = $(document.querySelector('#' + targetId));
            isAgile = true;
        }

        var $attachmentModule = $context && ($context.attr("id") === targetId) && $('.mod-content:not(.smart-attachment)', $context);

        if ($attachmentModule && $attachmentModule.size() > 0) {
            $attachmentModule.addClass('smart-attachment');
            var issue = new Issue($context, isAgile);
            if (issue) {
                var watcher = new CategoryPanelWatcher($context[0], issue, isAgile);
                CategoryDataManager.register(watcher);
            } else {
                console.log('Unable get issueKey');
            }
        }
    }

    function createRequest($context) {
        var isAgile = false;
        var targetId = DEFAULT_TARGET_ID;
        if (document.querySelector('#gh')) {
            targetId = 'ghx-tab-com-stiltsoft-jira-smart-attachments-attachment-cat-issue-section-agile';
            $context = $(document.querySelector('#' + targetId));
            isAgile = true;
        }

        var $attachmentModule = $context && ($context.attr("id") === targetId) && $('.mod-content:not(.smart-attachment)', $context);

        if ($attachmentModule && $attachmentModule.size() > 0) {
            $attachmentModule.addClass('smart-attachment');
            var issue = new Issue($context, isAgile);
            if (issue) {
                CategoryDataManager.makeRequest(issue.getIssueKey());
            } else {
                console.log('Unable get issueKey');
            }
        }
    }

    function initAfterIssueRefreshed() {
        var element = document.getElementById(DEFAULT_TARGET_ID);
        if (element) {
            var notInitiliazed = element.querySelector('#not-initialized');
            if (notInitiliazed) {
                var modContent = element.querySelector('.mod-content');
                if (modContent) {
                    modContent.classList.remove('smart-attachment');
                    createCategoryPanelWatcher($(element));
                }
            }
        }
    }

    var onReady = function () {
        ExecutorInstaller.init();
        JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function (e, context) {
            createCategoryPanelWatcher(context);
            //createRequest(context);
        });

        JIRA.bind(JIRA.Events.ISSUE_REFRESHED, function () {
            console.log('on issue refreshed');
            initAfterIssueRefreshed();
        });

        createCategoryPanelWatcher($('#attachment-cat-issue-section'));
    };

    if ($.isReady) {
        onReady()
    } else {
        AJS.toInit(onReady);
    }
});
