define('smart-attachments/service-desk-integration/CommentDialogDropZone', [
    'underscore',
    'jquery',
    'jira/util/events',
    'smart-attachments/base/CommonClasses',
    'smart-attachments/base/AutoDistribution',
    'smart-attachments/toolbox',
    'smart-attachments/dropzone/CategoryZones',
    'dndattachment/dropzones/CommentDialogDropZone'
], function (_, $, Events, CommonClasses, AutoDistribution, Toolbox, CategoryZones, CommentDialogDropZone) {

    var init = CommentDialogDropZone.prototype.init;
    var handleFilesReceived = CommentDialogDropZone.prototype.handleFilesReceived;
    var onDragLeave = CommentDialogDropZone.prototype.onDragLeave;
    var attachFile = CommentDialogDropZone.prototype.attachFile;

    var selectCategoryTemplate = SmartAttachment.Template.serviceDeskCategorySelect;
    var selectItemCategoryTemplate = SmartAttachment.Template.selectItemCategory;

    _.extend(CommentDialogDropZone.prototype, {
        init: function (element, uploadLimit) {
            this.issue = require('jira/issue');
            this.categoryConfig = [];
            this.invokedFromCategory = false;
            this.initDeferred = $.Deferred();

            var marker = this.getCategoryMarker(element);
            var args = arguments;

            Toolbox.fetchCategories(this.issue.getIssueId())
                .then((function (categoryConfig) {
                    init.apply(this, args);
                    this.finishInit(categoryConfig, marker);
                }).bind(this));
        },

        getCategoryMarker: function (element) {
            var marker = sessionStorage.getItem('invokedFromCategory');
            if (marker) {
                var dialog = {
                    content: $(element).closest('section'),
                    dropzone: this
                };
                this.invokedFromCategory = true;
                $(document).trigger('dialogContentReady-fake', [dialog]);
            }
            sessionStorage.removeItem('invokedFromCategory');
            return marker;
        },

        restorePrototypeMethods: function () {
            console.log("No categories support");
            this.handleFilesReceived = handleFilesReceived;
            this.onDragLeave = onDragLeave;
            this.attachFile = attachFile;
        },

        finishInit: function (categoryConfig, categoryMarker) {
            this.categoryConfig = categoryConfig;

            if (categoryConfig && categoryConfig.categories.length !== 0) {
                if (!categoryMarker) {
                    this.targetCategoryId = null;
                } else {
                    this.targetCategoryId = parseInt(categoryMarker);
                    this.dropCategoryId = this.targetCategoryId;
                }
                this.attachListeners();
            } else {
                this.restorePrototypeMethods();
            }

            this.$containerNode.prop('_decoratedDropZone', this);

            this.initDeferred.resolve({});
        },

        attachListeners: function () {
            console.log("attach to events");
            this.bind('uploadFinished', this.proxyUploadFinished.bind(this));
            var updateWarningStatusHandler = this.updateWarningStatus.bind(this);
            this.bind('progressBarDestroyed', updateWarningStatusHandler);
            this.bind('progressBarCategoryChange', updateWarningStatusHandler);

            if (!this.invokedFromCategory) {
                if (this.categoryConfig.dropZones) {
                    this.bind('change:targetCategoryId', this.showCategoryFiles.bind(this));
                } else {
                    this.bind('change:targetCategoryId', this.changeFilesCategory.bind(this));
                }
                this.renderSelectCategories();
                if (this.categoryConfig.dropZones) {
                    this.renderCategoryZones();
                }
            }
        },

        getCategoryById: function (id) {
            var value = parseInt(id);
            return _.find(this.categoryConfig.categories, function (category) {
                return category.id === value;
            });
        },

        getDefaultOrFirstAvailableCategoryId: function () {
            var defaultCategory = this.getCategoryById(0);
            return defaultCategory ? 0 : this.categoryConfig.categories[0].id;
        },

        detectCategoryId: function (progressBar) {
            var file = progressBar.file;
            var categoryId = file.categoryId;
            if (typeof categoryId === 'undefined') {
                categoryId = this.dropCategoryId || this.getDefaultOrFirstAvailableCategoryId();
            }
            if (categoryId === 'auto') {
                var matcher = AutoDistribution.createMatcher(categoryId, this.categoryConfig.categories);
                categoryId = matcher.match(file);
            }
            return categoryId;
        },

        proxyUploadFinished: function (e, progressBar) {
            var categories = this.categoryConfig.categories;
            var categoryId = this.detectCategoryId(progressBar);
            progressBar.category = this.getCategoryById(categoryId);
            if (this.categoryConfig.dropZones) {
                var $selectItemCategory = $(selectItemCategoryTemplate({
                    categories: categories,
                    selectedId: categoryId
                }));

                var dropZoneInstance = this;
                $selectItemCategory.find('select').on('change', function () {
                    var $target = $(this);
                    var categoryId = parseInt($target.val());
                    console.log('Move to category: ' + categoryId);
                    progressBar.category = dropZoneInstance.getCategoryById(categoryId);
                    var showAll = (dropZoneInstance.targetCategoryId === null);
                    if (!showAll && dropZoneInstance.targetCategoryId !== categoryId) {
                        progressBar.$node.hide();
                    }
                    dropZoneInstance.trigger('progressBarCategoryChange', progressBar);
                });
                progressBar.$node.find('.temp-upload-progress-bar__parent-container').append($selectItemCategory);
            }
            this.updateWarningStatus();
        },

        updateWarningStatus: function () {
            if (this.$warning) {
                var restricted = _.some(this.progressBars, function (progressBar) {
                    var category = progressBar.category;
                    return category.hasPermission && !category.allowOnPortal;
                });

                if (restricted) {
                    this.$warning.removeClass('hidden');
                } else {
                    this.$warning.addClass('hidden');
                }
            }
        },

        handleFilesReceived: function (files) {
            var args = arguments;
            this.initDeferred.then((function () {
                handleFilesReceived.apply(this, args)
            }).bind(this));
        },

        showCategoryFiles: function () {
            var showAll = this.targetCategoryId === null;
            _.each(this.progressBars, function (progressBar) {
                if (showAll || progressBar.category.id === this.targetCategoryId) {
                    progressBar.$node.show();
                } else {
                    progressBar.$node.hide();
                }
            }, this);
        },

        changeFilesCategory: function () {
            var category = this.getCategoryById(this.targetCategoryId);
            if (category) {
                _.each(this.progressBars, function (progressBar) {
                    progressBar.category = category;
                })
            }
            this.updateWarningStatus();
        },

        onSelectCategoryChange: function () {
            var targetCategoryId = this.$selectCategory.find('option:selected').val();
            if (targetCategoryId === "") {
                this.targetCategoryId = null;
            } else {
                this.targetCategoryId = parseInt(targetCategoryId);
            }
            this.trigger('change:targetCategoryId');
        },

        renderSelectCategories: function () {
            var portalRestriction = [];

            _.each(this.categoryConfig.categories, function (category) {
                if (category.hasPermission && !category.allowOnPortal) {
                    portalRestriction.push(category)
                }
            });

            var $selectBody = $(selectCategoryTemplate({
                allOption: this.categoryConfig.dropZones,
                categories: this.categoryConfig.categories,
                defaultId: this.targetCategoryId,
                portalRestriction: portalRestriction
            }));
            $selectBody.on('change', this.onSelectCategoryChange.bind(this));

            this.$selectCategory = $selectBody;

            if (portalRestriction.length) {
                this.$warning = $selectBody.find('span.category-warning');
            }
            this.$node.closest('#attachment-area').before($selectBody);
            // update size of a rich-editor
            if (this.getContainerNode().find('rich-editor').length) {
                var editorregistry = require("jira/editor/registry");
                editorregistry.on('register', (function (entry) {
                    this.getContainerNode().find('iframe').css('height', 102);
                }).bind(this));
            }

            this.getContainerNode().find('textarea').css('min-height', 130);
        },

        getCategoriesForRender: function () {
            if (this.categoryConfig.categories.length === 1 || !this.categoryConfig.dropZones) {
                return [];
            }

            var categoriesForRender = this.categoryConfig.categories.slice();

            if (AutoDistribution.isNeedToShowAutoDistributionZone(categoriesForRender)) {
                categoriesForRender.unshift(AutoDistribution.getAutoDistributionCategoryTemplate());
            }
            return categoriesForRender;
        },

        renderCategoryZones: function () {
            var maskElement = this.getContainerNode().find('.attach-files-drop-zone__dragover-mask')[0];
            var categoriesForRender = this.getCategoriesForRender();
            var isDropZones = this.categoryConfig.dropZones;
            this.categoryZones = new CategoryZones(maskElement, categoriesForRender);
            this.categoryZones.bind('dropToCategory', (function (event, categoryId) {
                if (isDropZones) {
                    this.dropCategoryId = categoryId;
                    this.setTargetCategory(null);
                } else {
                    this.dropCategoryId = this.targetCategoryId;
                }
                onDragLeave.call(this);
            }).bind(this));
            this.categoryZones.bind('leaveCategories', onDragLeave.bind(this));
        },

        onDragLeave: function () {
            if (!this.categoryConfig.dropZones) {
                onDragLeave.apply(this, arguments);
            }
        },
        setTargetCategory: function (value) {
            this.targetCategoryId = value;
            this.$selectCategory.find('option:selected').removeAttr('selected');
            this.$selectCategory.find('select option[value=' + value + ']').attr('selected', true).change();
        },

        getTempAttachments: function () {
            return this.progressBars.map(function (progressBar) {
                return new CommonClasses.TempAttachment(progressBar.getFileName(), progressBar.category.id);
            });
        },

        attachFile: function (fileIDs) {
            var ctx = this;
            var result = $.Deferred();

            if (!this.progressBars) {
                result.reject("There are no files");
                return result;
            }

            result.then(function () {
                Events.trigger('change:attachments', [ctx.issue]);
            });

            Toolbox.attachFilesToCategories(this.issue.getIssueId(), this.getTempAttachments())
                .then(function () {
                    var attachDeferred = attachFile.call(ctx, fileIDs);
                    attachDeferred.then(result.resolve);
                    attachDeferred.fail(result.reject);
                })
                .fail(function () {
                    result.reject("Save smart attachment failed");
                });
            return result;
        }
    });

    return CommentDialogDropZone;
});

AJS.toInit(function () {
    var globalDialogFactory = AJS.dialog2;
    if (!AJS.dialog2) {

    }
    globalDialogFactory.on("hide", function () {
        // хрязный хак для удаления стилей оверлеев используемых для загрузки файлов, корни растут из файла draggable.js
        // который подсчитывает dragLeave, dragEnter над .issue-container и сам скрывает/показывает оверлеи.
        // Вот и получается коллизия, когда за нас решили, что показать или скрыть.
        // Решение - отключение обработчика над issue-container (см attachment-drop-zone.js) и самостоятельное удаление
        // стилей по закрытию диалога.
        // В слачае если диалог не показывается удаление стилей идет штатными средствами из attachment-drop-zone.js
        var $body = $(document.body);
        $body.removeClass("attachments-drop-zone__window-dragover");
        $body.removeClass("attachments-drop-zone__dragover");
    });
});